from unittest import TestCase

from mystix.language.evaluation.main import run_program
from mystix.language.parsing import ParseError
from mystix.language.evaluation.errors import LanguageError


class EndToEndTests(TestCase):

    def test_undefined_source(self):
        code, err = run_program("tests/res/programs/e2e_undefined_source.mstx")
        self.assertEqual(1, code)
        self.assertIsInstance(err, LanguageError)

    def test_invalid_command(self):
        code, err = run_program("tests/res/programs/e2e_comment_command.mstx")
        self.assertEqual(2, code)
        self.assertIsInstance(err, ParseError)

    def test_nexist_file(self):
        code, err = run_program("tests/res/programs/e2e_does_not_exist.mstx")
        self.assertEqual(3, code)
        self.assertIsInstance(err, FileNotFoundError)



from flask import Flask, request, make_response
import json
import math


app = Flask(__name__)

time = range(1, 100000)
height = [2*t for t in time]
leaves = [math.sin(t) for t in time]


@app.route('/')
def hello_world():
    date = request.args.get('date')
    year = int(date[:4])
    month = int(date[5:7])
    day = int(date[9:11])
    index = (360 * (year - 2020)) + (30 * month) + day
    data = json.dumps({
        'data': [{
            'date': time[index],
            'plant_height': height[index],
            'plant_leaves': int(abs(leaves[index] * 20))
        }]
    })
    res = make_response((data, 200))
    res.headers['Content-Type'] = 'application/json'
    return res


def run_simulate():
    app.run(host="localhost", port=8000)



from mystix.ui.util import read_program_file
from mystix.language.tokenization import Tokenizer, TokenizationError
from mystix.language.parsing import Parser, ParseError
from mystix.language.evaluation import Evaluator


def process(content: str, e: Evaluator, duration: int):
    try:
        t = Tokenizer(content)
        t.tokenize()
        p = Parser(t).parseProgram()
        return e.evaluate(p, duration=duration)
    except (TokenizationError, ParseError) as e:
        print("\nERROR: %s\n" % str(e))
        return 2, e


def run_program(filename: str, graphics=False, duration=5000):
    try:
        return read_program_file(filename, lambda c: process(c, Evaluator(
            graphics=graphics), duration=duration))
    except FileNotFoundError as e:
        print("\nERROR: Could not read file '%s'\n" % filename)
        return 3, e
